/* Copyright (c) 1997, MpegTV LLC */

#ifndef lint
static char rcsid[] = "$Header: /net/hopkins/var/home/tristan/mpeg/dec/RCS/ci_file.c,v 1.2 1997/08/21 11:51:26 tristan Exp $";
static char copyright[] = "Copyright (c) 1997, MpegTV LLC";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/misc.h"

#include "mtvp/sih_api.h"

/*
 * This is a working example of SIH plug-in.
 * See the README file.
 */

/*--------------------------------------------------------------------------+
| Macros
+--------------------------------------------------------------------------*/
/*
 * This SIH plugin only recognizes URL's starting with the MY_PROTOCOL string.
 */
 
/* In this example, we use "file:" to override the built-in "file:" handler */

#define MY_PROTOCOL "file:"
#define MY_PROTOCOL_SIZE (sizeof MY_PROTOCOL - 1)

/*--------------------------------------------------------------------------+
| Data types
+--------------------------------------------------------------------------*/
/* handler data */
struct mtvp_sih_handler_data {
	const MTVP_SIH_RESOURCES *sih_resources;
};

/* stream data */
struct mtvp_sih_stream_data {
	struct mtvp_sih_handler_data *handler_data;	/* should always be there */
	int		fd;
};

/*
 * In this example, our struct mtvp_sih_stream_data structure contains only a file descriptor,
 * in addition to the pointer to the struct mtvp_sih_handler_data (which should always be there).
 * The struct mtvp_sih_stream_data structure can contain absolutely anything you want,  The nature of
 * this structure is only known to the SIH routines defined in the SIH plug-in.
 */


/*--------------------------------------------------------------------------+
| _init
+--------------------------------------------------------------------------*/
int
_init()
{
	/*
	 * Unix only!
	 * If the library exports a routine named  _init(),  then  that
     * code  is  executed  before  dlopen()  returns.
     */

	printf("_init(): loading custom SIH handler\n");

	return 0;
}


/*--------------------------------------------------------------------------+
| _fini
+--------------------------------------------------------------------------*/
int
_fini()
{
	/*
	 * Unix only!
	 * If the dynamic library exports a routine named _fini(),
	 * then that routine is called just before the library is unloaded.
	 */

	printf("_fini(): un-loading custom SIH handler\n");

	return 0;
}

/*--------------------------------------------------------------------------+
| SIH_init
+--------------------------------------------------------------------------*/
static int
SIH_init(
	const MTVP_SIH_RESOURCES *sih_resources,
	const char *p,
	struct mtvp_sih_handler_data **handler_data_ptr
)
{
	struct mtvp_sih_handler_data *handler_data;

	printf("SIH_init with \"%s\"\n", p);

	/* allocate storage for our opaque struct mtvp_sih_handler_data structure */
	handler_data = (struct mtvp_sih_handler_data *)calloc(1, sizeof(struct mtvp_sih_handler_data));

	if (handler_data == NULL) {
		perror("SIH_init");
		return errno;
	}

	handler_data->sih_resources = sih_resources;

	/*
	 * Return a pointer to our struct mtvp_sih_handler_data structure.
	 * Note that handler_init is defined as returning a (void *) pointer,
	 * because the returned pointer is opaque, i.e. struct mtvp_sih_handler_data is unknown
	 * from the caller.
	 */
	*handler_data_ptr = handler_data;
	return 0;
}


/*--------------------------------------------------------------------------+
| SIH_probe_url
+--------------------------------------------------------------------------*/
static int
SIH_probe_url(
	const MTVP_SIH_RESOURCES *sih_resources,
	const char *url
)
{
	if (strncmp(MY_PROTOCOL, url, MY_PROTOCOL_SIZE) != 0) {
		printf("SIH_probe_url: URL %s cannot be opened by this handler\n", url);

		return EINVAL;
	}

	printf("SIH_probe_url: URL %s can be opened by this handler\n", url);

	return 0;
}

/*--------------------------------------------------------------------------+
| SIH_open_url
+--------------------------------------------------------------------------*/
static int
SIH_open_url(
	struct mtvp_sih_handler_data *handler_data,	/* pointer that was returned by SIH_init */
	const char *url,	/* assumed to be a file:filename in this example */
	struct mtvp_sih_stream_data **stream_data_ptr
)
{
	struct mtvp_sih_stream_data *stream_data;
	const char *filename;
	int		fd;

	/*
	 * The parameter of SIH_open_url is an URL passed by the application
	 * to the COM_SIH_OPEN_URL command.  The URL identifies the stream
	 * to be opened.
	 *
	 * SIH_open_url returns a pointer to an opaque object that
	 * identifies the stream, i.e.
	 * only your SIH plug-in known about the structure of the
	 * opaque object.  If the specified stream cannot be opened,
	 * SIH_open_url returns NULL and sets errno.
	 */

	filename = url + strlen(MY_PROTOCOL);

	printf("SIH_open_url calls open(\"%s\", O_RDONLY)\n", filename);

	/* check that sih_feedback() works */
	(*handler_data->sih_resources->feedback)(handler_data->sih_resources, url, strlen(url) + 1);

	/* in this example, we just open the specified file */
	fd = open(filename, O_RDONLY);

	if (fd == -1) {
		perror("SIH_open_url");
		return errno;
	}

	/* allocate storage for our opaque struct mtvp_sih_stream_data structure */
	stream_data = (struct mtvp_sih_stream_data *)calloc(1, sizeof(struct mtvp_sih_stream_data));

	if (stream_data == NULL) {
		perror("SIH_open_url");
		return errno;
	}

	stream_data->handler_data = handler_data;

	/* the file descriptor for the opened file is stored in our struct mtvp_sih_stream_data structure */
	stream_data->fd = fd;
        
	/*
	 * Return a pointer to our PLUGIN_DATA structure.
	 * Note that SIH_open_url is defined as returning a (void *) pointer,
	 * because the returned pointer is opaque, i.e. struct mtvp_sih_stream_data is unknown
	 * from the caller (mtvp).
	 */
	*stream_data_ptr = stream_data;
	return 0;
}

/*--------------------------------------------------------------------------+
| SIH_fd
+--------------------------------------------------------------------------*/
static int
SIH_fd(
	struct mtvp_sih_stream_data *stream_data,
	void **fd_ptr
)
{
	/*
	 * Returns the file descriptor associated with the stream.
	 * 
	 * This routine must return a real file descriptor.
	 * The Player calls a blocking select on this file descriptor
	 * and if there is something available for reading on it, calls SIH_read
	 * (provided that there is buffer space available to read some input data).
	 * 
	 * After calling SIH_fd, mtvp calls fcntl with F_SETFL
	 * to set O_NONBLOCK on the fd.
	 */

	printf("SIH_fd returns %d\n", stream_data->fd);

	*fd_ptr = (void *)stream_data->fd;
	
	return 0;
}


/*--------------------------------------------------------------------------+
| SIH_close
+--------------------------------------------------------------------------*/
static void
SIH_close(
	struct mtvp_sih_stream_data *stream_data
)
{
	/*
	 * Close the stream.
	 * 
	 * This routine must close every fd opened by SIH_open_url and free the PLUGIN_DATA
	 * structure.  All the resources allocated for the stream should be released.
	 * 
	 * After a call to SIH_close, the "void *stream"
	 * should be considered invalid.
	 */

	printf("SIH_close\n");

	close(stream_data->fd);

	free(stream_data);
}

/*--------------------------------------------------------------------------+
| SIH_free
+--------------------------------------------------------------------------*/
static void
SIH_free(
	struct mtvp_sih_handler_data *handler_data
)
{
	printf("SIH_free\n");

	free(handler_data);
}


/*--------------------------------------------------------------------------+
| SIH_read
+--------------------------------------------------------------------------*/
static int
SIH_read(
	struct mtvp_sih_stream_data *stream_data,
	char *buf,
	size_t count,
	size_t *count_read
)
{
	int		n;

	/*
	 * SIH_read must behave exactly like the
	 * regular read routine when used on a fd
	 * that has O_NONBLOCK set.
	 * 
	 * If data becomes available for reading on the fd returned by SIH_fd,
	 * SIH_read is called. Note that mtvp may call SIH_read at any time, even
	 * when there is no data available on the fd.
	 * 
	 * SIH_read should always behave like a non-blocking read
	 * (i.e. like read behaves when the flag O_NONBLOCK set).
	 * This means that SIH_read should never block, even
	 * if there is not enough data available.
	 * 
	 * If there is no data available,
	 * SIH_read should return -1 with errno set to EAGAIN.
	 * 
	 * If there is less data available than requested,
	 * SIH_read should copy all the data available into the
	 * buffer and return the number of bytes copied.
	 * 
	 * SIH_read returning 0 means EOF.
	 * 
	 * SIH_read returning anything other than MTVP_SIH_READ_ERROR_TRY_AGAIN
	 * or MTVP_SIH_READ_ERROR_TOO_SMALL means an error occured.  All read errors
	 * are considered like EOF.
	 *
	 * MTVP_SIH_READ_ERROR_TRY_AGAIN will cause SIH_read to be called again
	 * with the same parameters.
	 *
	 * MTVP_SIH_READ_ERROR_TOO_SMALL will cause SIH_read to be called again
	 * on the next buffer (i.e. a larger buffer, if the current buffer
	 * was partially filled).
	 */

	n = read(stream_data->fd, buf, count);

	printf("SIH_read calls read(%d, buf, %d), returns %d\n", stream_data->fd, count, n);

#if 0
	/* uncomment this to check that feedback() works in SIH_read */
	{
		const char buf[] = "calling feedback in SIH_read";
		(*stream_data->handler_data->sih_resources->feedback)(stream_data->handler_data->sih_resources, buf, sizeof buf);
	}
#endif

	if (n < 0) {
		int status = errno; if (status == 0) status = -1;
		if (status == EINTR || status == EAGAIN) {
			return MTVP_SIH_READ_ERROR_TRY_AGAIN;
		}
		return status;
	}

	*count_read = n;
	return 0;
}



/*--------------------------------------------------------------------------+
| SIH_seek
+--------------------------------------------------------------------------*/
static int
SIH_seek(
	struct mtvp_sih_stream_data *stream_data,
	off_t offset
)
{
	off_t	n;

	/*
	 * SIH_lseek must behave exactly like the regular lseek routine.
	 * 
	 * In particular:
	 * 
	 * SIH_lseek should return -1 with errno set to ESPIPE
	 * if the input does not support seeking.
	 */
	 
	n = lseek(stream_data->fd, offset, SEEK_SET);

	printf("SIH_lseek calls lseek(%d, %ldL, %d), returns %ld\n", stream_data->fd, (long)offset, whence, (long)n);

	if (n < 0) {
		return errno;
	}

	return 0;
}


/*--------------------------------------------------------------------------+
| SIH_size
+--------------------------------------------------------------------------*/
static int
SIH_size(
	struct mtvp_sih_stream_data *stream_data,
	off_t *size_ptr
)
{
	struct	stat stat_buf;
	off_t	size;
	int		n;

	/*
	 * SIH_size, if defined, must return the size of the file, in bytes.
	 * 
	 * If SIH_size is not defined, appropriate calls to SIH_lseek are used
	 * to determine the size of the file.
	 */
	 
	n = fstat(stream_data->fd, &stat_buf);

	printf("SIH_size calls fstat(%d, ...), returns %d, size = %lu\n", stream_data->fd, n, (u_long)stat_buf.st_size);

	if (n != 0) {
		return errno;
	}

	*size_ptr = stat_buf.st_size;
	return 0;
}


/*--------------------------------------------------------------------------+
| SIH_cntl
+--------------------------------------------------------------------------*/
static int
SIH_cntl(
	struct mtvp_sih_handler_data *handler_data,
	const MTVP_SIH_CNTL *data,
	u_int		size
)
{
	/*
	 * SIH_cntl is called if and only if the application sends
	 * the message MTVP_COMMAND_SIH_CNTL.  The parameter "data" is a pointer to
	 * an arbitrary chunk of data specified by the application.
	 * "size" is the size of the chunk of data.
	 */
	 
	/*
	 * In this example, the data block passed to SIH_cntl() is known to be
	 * a string, so we just print it to show that we received it.
	 */
	 
	printf("SIH_cntl: data = \"%s\", size = %u\n", (const char *)data->data, size);

	return 0;
}

/*--------------------------------------------------------------------------+
| SIH_register
+--------------------------------------------------------------------------*/
int
SIH_register(MTVP_SIH_EXPOSED *exposed, int version)
{
	if (version == MTVP_SIH_VERSION) {

	 	exposed->SIH_version = MTVP_SIH_VERSION;
		exposed->SIH_init = SIH_init;
		exposed->SIH_free = SIH_free;
		exposed->SIH_probe_url = SIH_probe_url;
		exposed->SIH_open_url = SIH_open_url;
		exposed->SIH_close = SIH_close;
		exposed->SIH_read = SIH_read;
		exposed->SIH_seek = SIH_seek;
		exposed->SIH_cntl = SIH_cntl;
		exposed->SIH_fd = SIH_fd;
		exposed->SIH_size = SIH_size;
		exposed->SIH_buffer = NULL;
		exposed->SIH_notify_free = NULL;
		exposed->SIH_notify_rate = NULL;
	}
	return MTVP_SIH_VERSION;
}

/* eof */
