#ifndef lint
static const char rcsid[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/voh_x.c,v 1.246 1999/03/20 12:01:19 tristan Exp tristan $";
static const char copyright[] = "Copyright (c) 1997, MpegTV LLC";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/misc.h"
#include "mtvp/dith_im.h"
#include "mtvp/msg_types.h"
#include "mtvp/voh_api.h"

#include "SDL.h"
#include "SDL_syswm.h"

/*--------------------------------------------------------------------------+
| Configuration
+--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------+
| Macros
+--------------------------------------------------------------------------*/
#undef MTVP_MSG_PTR
#define MTVP_MSG_PTR	(((struct mtvp_voh_data *)handler_data)->voh_resources->msg)


/*--------------------------------------------------------------------------+
| Data types
+--------------------------------------------------------------------------*/


typedef enum {
	SDL_BPP_MODE_BEST = 0,
	SDL_BPP_MODE_16 = 1,
	SDL_BPP_MODE_24 = 2,
	SDL_BPP_MODE_32 = 3
} SDL_BPP_MODE;


struct mtvp_voh_data {
	SDL_BPP_MODE sdl_bpp_mode;		/* bpp mode (best or specified bpp) */
	int		sdl_bpp;				/* bits per pixel */
	int		sdl_video_flags;		/* SDL video flags */
	Bool	sdl_flip;				/* if True use SDL_Flip */
	SDL_Surface *sdl_screen;		/* NULL or ptr to SDL screen */
	
	MTVP_VOH_DITHERED_IMAGE *voh_dithered_image;
	const MTVP_VOH_RESOURCES *voh_resources;
	MTVP_VOH_CONTROL 	voh_control;
};


/*--------------------------------------------------------------------------+
| VOH_dithered_image_show
+--------------------------------------------------------------------------*/
static void
VOH_dithered_image_show(
	struct mtvp_voh_data *handler_data
)
{
	u_int	x;
	u_int	y;

	MTVP_MSG(MTVP_MSG_X, ("X: showing dithered image"));

  	if (handler_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}
	
	if (handler_data->sdl_flip) {
		/* printf("SDL_Flip...\n"); */
		if (SDL_Flip(handler_data->sdl_screen) < 0) {
			abort_now();
		}
	} else {
		if (SDL_MUSTLOCK(handler_data->sdl_screen)) {
			/* printf("SDL_UnlockSurface...\n"); */
			SDL_UnlockSurface(handler_data->sdl_screen);
			handler_data->voh_dithered_image->array = NULL;
		}
	}
}


/*--------------------------------------------------------------------------+
| VOH_window_create
+--------------------------------------------------------------------------*/
static int
VOH_window_create (
	struct mtvp_voh_data *handler_data,
	MTVP_DITHER_TYPE dither_type,
	const MTVP_VOH_SIZE *voh_size,
	MTVP_VOH_WINDOW	existing_window,
	const MTVP_VOH_CONTROL *voh_control,
	u_long	flags,
	void **fd_ptr_returned	
)
{
	int     screen;
	const SDL_VideoInfo *video_info;

  	if (handler_data->sdl_screen != NULL) {
		abort_now(); /* should never happen */
	}

	if (flags & MTVP_VOH_CONTROL_PEL_ASPECT_RATIO) {
		/* aspect ratio handling not implemented */
	}
			
	if (dither_type != MTVP_DITHER_COLOR) {
		printf("mtvp: fullscreen can only be used with full-color video\n");
		printf("mtvp: restart the X server with 16-bit, 24-bit or 32-bit depth\n");
		printf("mtvp: with: startx -- -bpp <depth> [e.g. startx -- -bpp 16]\n");				printf("mtvp: and make sure that video mode is set to full-color\n");
		goto err;
	}

	/* Inizialize the SDL library */
	MTVP_MSG(MTVP_MSG_X, ("X: SDL: calling SDL_Init"));
	if (SDL_Init(SDL_INIT_VIDEO) < 0) {
		printf("mtvp: Couldn't initialize SDL: %s\n", SDL_GetError());
		printf("mtvp: Make sure you have /usr/X11R6/lib/libSDLx11.so.0.9\n");
		printf("mtvp: This library (SDL) can be downloaded from the MpegTV FTP site\n");
		printf("mtvp: ftp://ftp.mpegtv.com/pub/mpeg/mpegtv/player\n");
		goto err;
	}
	MTVP_MSG(MTVP_MSG_X, ("X: SDL: SDL_Init succeeded"));
	SDL_EventState(SDL_ACTIVEEVENT, SDL_IGNORE);
	SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);
	video_info = SDL_GetVideoInfo();

	MTVP_MSG(MTVP_MSG_X, ("X: SDL: best video mode is %u-bit surface", video_info->vfmt->BitsPerPixel));
	
	MTVP_MSG(MTVP_MSG_X, ("X: sdl_bpp_mode = %u", handler_data->sdl_bpp_mode));
	
	switch (handler_data->sdl_bpp_mode) {
		case SDL_BPP_MODE_BEST:
			handler_data->sdl_bpp = video_info->vfmt->BitsPerPixel;
			break;
		case SDL_BPP_MODE_16:	handler_data->sdl_bpp = 16; break;
		case SDL_BPP_MODE_24:	handler_data->sdl_bpp = 24; break;
		case SDL_BPP_MODE_32:	handler_data->sdl_bpp = 32; break;
		default:				abort_now(); break;
	}

	MTVP_MSG(MTVP_MSG_X, ("X: SDL: will use a %u-bit surface", handler_data->sdl_bpp));

	if (handler_data->sdl_bpp != 16 && handler_data->sdl_bpp != 24 && handler_data->sdl_bpp != 32) {
		MTVP_MSG(MTVP_MSG_X, ("X: SDL: unsupported BitsPerPixel %u", handler_data->sdl_bpp));
		printf("mtvp: cannot use fullscreen when the X server depth is 8-bit\n");
		printf("mtvp: restart the X server with 16-bit, 24-bit or 32-bit depth\n");
		printf("mtvp: with: startx -- -bpp <depth> [e.g. startx -- -bpp 16]\n");
		goto err;
	}

	{
		SDL_SysWMinfo info;

		SDL_VERSION(&info.version);
		if (SDL_GetWMInfo(&info) < 0) {
			MTVP_MSG(MTVP_MSG_X, ("X: SDL: SDL_GetWMInfo failed: %s", SDL_GetError()));
			goto err;
		}
		/* connextion number (fd) to X display used by SDL */
//		(*handler_data->voh_resources->notify_fd)(handler_data->voh_resources, ConnectionNumber(info.display));
		*fd_ptr_returned = (void *)ConnectionNumber(info.display);
	}

			
	MTVP_MSG(MTVP_MSG_X, ("X: SDL: calling SDL_SetVideoMode"));
	handler_data->sdl_screen = SDL_SetVideoMode(voh_size->frame_width, voh_size->frame_height, handler_data->sdl_bpp, handler_data->sdl_video_flags);

	if (handler_data->sdl_screen == NULL) {
		MTVP_MSG(MTVP_MSG_X, ("X: SDL: Couldn't initialize video mode: %s", SDL_GetError()));
		printf("mtvp: Couldn't set SDL video mode: %s\n", SDL_GetError());
		printf("mtvp: make sure that XFree86 is configured to support at least one\n");
		printf("mtvp: resolution equal or larger than the video size of this stream.\n");
		goto err;
	}
	MTVP_MSG(MTVP_MSG_X, ("X: SDL: SDL_SetVideoMode succeeded"));

	assert(handler_data->sdl_screen->format->BitsPerPixel == 16
		|| handler_data->sdl_screen->format->BitsPerPixel == 24
		|| handler_data->sdl_screen->format->BitsPerPixel == 32);

	if (handler_data->sdl_video_flags & SDL_FULLSCREEN) {
		if (handler_data->sdl_screen->flags & SDL_FULLSCREEN) {
			MTVP_MSG(MTVP_MSG_X, ("X: SDL: screen supports full screen"));
		} else {
			uid_t	euid = geteuid();

			MTVP_MSG(MTVP_MSG_X, ("X: SDL: screen doesn't support full screen"));
			printf("mtvp: cannot use fullscreen\n");
			
#if 0 /* Tristan Savatier - Mon Dec 14 03:39:17 PST 1998 */
//					printf("euid = %d\n", (int)euid);
#endif /* 0 - Tristan Savatier - Mon Dec 14 03:39:17 PST 1998 */
			if (euid != 0) {
				printf("mtvp: You must be root (or mtvp must be set-uid root) to use fullscreen\n");
				printf("mtvp: For set-uid root, become root and use the following commands:\n");
				printf("mtvp: $ chown root mtvp; chmod +s mtvp\n");
				printf("mtvp: Then restart the program and try again...\n");
				printf("mtvp: Note: set-uid programs may create security holes\n");
			}
			printf("mtvp: For details, read the README-Fullscreen file.\n");
#if 0 /* Tristan Savatier - Sun Mar 21 23:57:26 PST 1999 */
//			goto err;
#endif /* 0 - Tristan Savatier - Sun Mar 21 23:57:26 PST 1999 */
		}
	}
	if (handler_data->sdl_screen->flags & SDL_HWSURFACE) {
		MTVP_MSG(MTVP_MSG_X, ("X: SDL: screen supports hardware surface"));
		if (handler_data->sdl_screen->flags & SDL_DOUBLEBUF) {
			MTVP_MSG(MTVP_MSG_X, ("X: SDL: screen supports double buffering"));
			handler_data->sdl_flip = True;
		} else {
			MTVP_MSG(MTVP_MSG_X, ("X: SDL: screen doesn't support double buffering"));
			handler_data->sdl_flip = False;
		}
	} else {
		MTVP_MSG(MTVP_MSG_X, ("X: SDL: screen doesn't support hardware surface"));
		handler_data->sdl_flip = True;
	}
		
#if 0 /* Tristan Savatier - Sun Mar 21 23:08:49 PST 1999 */
//	handler_data->depth = handler_data->sdl_screen->format->BitsPerPixel;
#endif /* 0 - Tristan Savatier - Sun Mar 21 23:08:50 PST 1999 */
				
#if 0 /* Tristan Savatier - Mon Dec 14 03:31:38 PST 1998 */
//			printf("handler_data->sdl_fd = %d\n", handler_data->sdl_fd);
//			printf("handler_data->sdl_screen->flags = 0x%08x\n", handler_data->sdl_screen->flags);
//			printf("SDL_HWSURFACE = %d\n", !!(handler_data->sdl_screen->flags & SDL_HWSURFACE));
//			printf("SDL_THREADSAFE = %d\n", !!(handler_data->sdl_screen->flags & SDL_THREADSAFE));
//			printf("SDL_ANYFORMAT = %d\n", !!(handler_data->sdl_screen->flags & SDL_ANYFORMAT));
//			printf("SDL_HWPALETTE = %d\n", !!(handler_data->sdl_screen->flags & SDL_HWPALETTE));
//			printf("SDL_DOUBLEBUF = %d\n", !!(handler_data->sdl_screen->flags & SDL_DOUBLEBUF));
//			printf("SDL_FULLSCREEN = %d\n", !!(handler_data->sdl_screen->flags & SDL_FULLSCREEN));
//			printf("SDL_HWACCEL = %d\n", !!(handler_data->sdl_screen->flags & SDL_HWACCEL));
//			printf("handler_data->sdl_screen->format->BitsPerPixel = %d\n", handler_data->sdl_screen->format->BitsPerPixel);
//			printf("handler_data->sdl_screen->format->BytesPerPixel = %d\n", handler_data->sdl_screen->format->BytesPerPixel);
//			printf("handler_data->sdl_screen->format->Rloss = %d\n", handler_data->sdl_screen->format->Rloss);
//			printf("handler_data->sdl_screen->format->Gloss = %d\n", handler_data->sdl_screen->format->Gloss);
//			printf("handler_data->sdl_screen->format->Bloss = %d\n", handler_data->sdl_screen->format->Bloss);
//			printf("handler_data->sdl_screen->format->Aloss = %d\n", handler_data->sdl_screen->format->Aloss);
//			printf("handler_data->sdl_screen->format->Rshift = %d\n", handler_data->sdl_screen->format->Rshift);
//			printf("handler_data->sdl_screen->format->Gshift = %d\n", handler_data->sdl_screen->format->Gshift);
//			printf("handler_data->sdl_screen->format->Bshift = %d\n", handler_data->sdl_screen->format->Bshift);
//			printf("handler_data->sdl_screen->format->Ashift = %d\n", handler_data->sdl_screen->format->Ashift);
//			printf("handler_data->sdl_screen->format->Rmask = %d\n", handler_data->sdl_screen->format->Rmask);
//			printf("handler_data->sdl_screen->format->Gmask = %d\n", handler_data->sdl_screen->format->Gmask);
//			printf("handler_data->sdl_screen->format->Bmask = %d\n", handler_data->sdl_screen->format->Bmask);
//			printf("handler_data->sdl_screen->format->Amask = %d\n", handler_data->sdl_screen->format->Amask);
//			printf("handler_data->sdl_screen->w = %d\n", handler_data->sdl_screen->w);
//			printf("handler_data->sdl_screen->h = %d\n", handler_data->sdl_screen->h);
//			printf("handler_data->sdl_screen->pitch = %d\n", handler_data->sdl_screen->pitch);
#endif /* 0 - Tristan Savatier - Mon Dec 14 03:31:38 PST 1998 */

	return 0;

err:
	MTVP_MSG(MTVP_MSG_X, ("X: SDL: cannot use SDL"));
	handler_data->sdl_screen = NULL;

    MTVP_MSG(MTVP_MSG_X, ("X: SDL: calling SDL_Quit"));
	SDL_Quit();

	/* return error status - SDL does not provide any meaningfull error status... */
	return -1;
}


/*--------------------------------------------------------------------------+
| VOH_window_destroy
+--------------------------------------------------------------------------*/
static void
VOH_window_destroy (
	struct mtvp_voh_data *handler_data
)
{

  	if (handler_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}

    MTVP_MSG(MTVP_MSG_X, ("X: SDL: calling SDL_Quit"));
	SDL_Quit();
	
	handler_data->sdl_screen = NULL;
}

/*--------------------------------------------------------------------------+
| VOH_dithered_image_free
+--------------------------------------------------------------------------*/
static void
VOH_dithered_image_free (
	struct mtvp_voh_data *handler_data
)
{

	// free handler_data->voh_dithered_image->array
	// but nothing to free in that case
}


/*--------------------------------------------------------------------------+
| VOH_dithered_image_alloc
+--------------------------------------------------------------------------*/
static int
VOH_dithered_image_alloc(
	struct mtvp_voh_data *handler_data
)
{

	if (handler_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}
	handler_data->voh_dithered_image->array = handler_data->sdl_screen->pixels;
	handler_data->voh_dithered_image->bytes_per_pel = handler_data->sdl_screen->format->BytesPerPixel;
	handler_data->voh_dithered_image->width = handler_data->sdl_screen->w;
	handler_data->voh_dithered_image->pitch = handler_data->sdl_screen->pitch;
	handler_data->voh_dithered_image->height = handler_data->sdl_screen->h;

	return 0;
}



/*--------------------------------------------------------------------------+
| VOH_resolve_dither
+--------------------------------------------------------------------------*/
static int
VOH_resolve_dither(
	struct mtvp_voh_data *handler_data,
	MTVP_VOH_WINDOW	existing_window,
	Bool	dither_ok[MTVP_DITHER_COUNT],
	MTVP_DITHER_TYPE *prefered_type_ptr
)
{

	/* In this example, we only handle full-color dither */
	
	dither_ok[MTVP_DITHER_GRAY] = False;
	dither_ok[MTVP_DITHER_ORDERED] = False;
	dither_ok[MTVP_DITHER_COLOR] = True;
	
	*prefered_type_ptr = MTVP_DITHER_COLOR;
	
	return 0;
}


/*--------------------------------------------------------------------------+
| VOH_read_and_handle_all_pending_events
+--------------------------------------------------------------------------*/
static int
VOH_read_and_handle_all_pending_events(
	struct mtvp_voh_data *handler_data,
	Bool must_handle_at_least_one_event
)
{

	/*
	 * Return number of events processed, or -1 if an error occured.
	 */

	/* in this example, any event causes the application to exit */
	
  	if (handler_data->sdl_screen != NULL) {
  	
  		if (SDL_PollEvent(NULL) != 0) {
			printf("VOH_read_and_handle_all_pending_events: found event, exiting\n");
			
			/* in this case we restore the correct video settings and exit */
			SDL_Quit(); /* you should not do that in a real application, of course */
			exit(0);	/* you should not do that in a real application, of course */
  		}
  	}
  	return 0;
}


/*--------------------------------------------------------------------------+
| VOH_set_control
+--------------------------------------------------------------------------*/
static int
VOH_set_control(
	struct mtvp_voh_data *handler_data,
	MTVP_VOH_CONTROL *voh_control,
	u_long	flags
)
{

	if (flags & MTVP_VOH_CONTROL_PEL_ASPECT_RATIO) {
		/* aspect ratio change not implemented */
	}

	return 0;
}


/*--------------------------------------------------------------------------+
| VOH_cntl
+--------------------------------------------------------------------------*/
static int
VOH_cntl(
	struct mtvp_voh_data *handler_data,
	const MTVP_VOH_CNTL *ptr,
	u_int	size
)
{

	/*
	 * VOH_cntl is called if and only if the application sends
	 * the message COM_VOH_CNTL.  The parameter "data" is a pointer to
	 * an arbitrary chunk of data specified by the application.
	 * "size" is the size of the chunk of data.
	 */
	 
	/*
	 * In this example, the data block passed to VOH_cntl() is known to be
	 * a string, so we just print it to show that we received it.
	 */
	 
	printf("VOH_cntl: ptr = \"%s\", size = %u\n", (const char *)ptr, size);
	
	return 0;
}



/*--------------------------------------------------------------------------+
| VOH_get_cm_pixel_values
+--------------------------------------------------------------------------*/
static u_long *
VOH_get_cm_pixel_values(
	struct mtvp_voh_data *handler_data
)
{

	/* not implemented since we only handle full-color windows */
	return NULL;
}

/*--------------------------------------------------------------------------+
| VOH_get_rgb_masks
+--------------------------------------------------------------------------*/
static void
VOH_get_rgb_masks(
	struct mtvp_voh_data *handler_data,
    MTVP_VOH_RGB_MASK *masks
)
{

  	if (handler_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}

	masks[MTVP_VOH_RED].mask = handler_data->sdl_screen->format->Rmask;
	masks[MTVP_VOH_GREEN].mask = handler_data->sdl_screen->format->Gmask;
	masks[MTVP_VOH_BLUE].mask = handler_data->sdl_screen->format->Bmask;
}


/*--------------------------------------------------------------------------+
| VOH_free
+--------------------------------------------------------------------------*/
static void
VOH_free(
	struct mtvp_voh_data *handler_data
)
{

    MTVP_MSG(MTVP_MSG_X, ("X: releasing handler_data memory"));
	bzero(handler_data, sizeof *handler_data);	/* helps to catch bugs */
	free(handler_data);
}


/*--------------------------------------------------------------------------+
| VOH_dithered_image_lock
+--------------------------------------------------------------------------*/
static Bool
VOH_dithered_image_lock(
	struct mtvp_voh_data *handler_data
)
{

	/*
	 * Must return False if dithered image is displayed instantly when
	 * written into (i.e. hardware plane with no double buffer).
	 * Must return True if dithered image is displayed only after
	 * VOH_dithered_image_show has been called.
	 */
	
  	if (handler_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}

	if (SDL_MUSTLOCK(handler_data->sdl_screen)) {
		/* printf("SDL_LockSurface...\n"); */
		if (SDL_LockSurface(handler_data->sdl_screen) <0 ) {
			abort_now();
		}
		handler_data->voh_dithered_image->array = handler_data->sdl_screen->pixels;
	}	
	return handler_data->sdl_flip;
}

/*--------------------------------------------------------------------------+
| VOH_dithered_image_unlock
+--------------------------------------------------------------------------*/
static void
VOH_dithered_image_unlock(
	struct mtvp_voh_data *handler_data
)
{
	
  	if (handler_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}
		
	if (!handler_data->sdl_flip && SDL_MUSTLOCK(handler_data->sdl_screen)) {
		/* printf("SDL_UnlockSurface...\n"); */
		SDL_UnlockSurface(handler_data->sdl_screen);
		handler_data->voh_dithered_image->array = NULL;
	}
}

/*--------------------------------------------------------------------------+
| VOH_init
+--------------------------------------------------------------------------*/
static int
VOH_init(
	const MTVP_VOH_RESOURCES *voh_resources,
	MTVP_VOH_DITHERED_IMAGE *voh_dithered_image,
	const void *ptr,
	struct mtvp_voh_data **handler_data_returned
)
{
	struct mtvp_voh_data *handler_data;
	int	status;
	
    handler_data = (struct mtvp_voh_data *)calloc(1, sizeof(struct mtvp_voh_data));
    
    if (handler_data == NULL) {
		MTVP_MSG_ERROR (MTVP_MSG_X, ("X: cannot allocate handler_data"));
		status = errno;
		goto err;
    }

    handler_data->voh_resources = voh_resources;
    handler_data->voh_dithered_image = voh_dithered_image;    

	handler_data->sdl_bpp_mode = SDL_BPP_MODE_BEST;
	handler_data->sdl_video_flags = SDL_HWPALETTE|SDL_FULLSCREEN|SDL_DOUBLEBUF|SDL_HWSURFACE;


	*handler_data_returned = handler_data;
	
	return 0;
err:
	{		
		if (handler_data != NULL) {
			memory_free(handler_data);
		}		
		return status ? status : -1;
	}
}

/*--------------------------------------------------------------------------+
| VOH_set_clut
+--------------------------------------------------------------------------*/
static int
VOH_set_clut(
	struct mtvp_voh_data *handler_data
)
{

	/* Color LookUp Table (palette) handling not implemented in this example */

	return 0;
}

/*--------------------------------------------------------------------------+
| VOH_free_clut
+--------------------------------------------------------------------------*/
static void
VOH_free_clut(
	struct mtvp_voh_data *handler_data
)
{

	/* Color LookUp Table (palette) handling not implemented in this example */
}

/*--------------------------------------------------------------------------+
| VOH_get_clut_pixel_values
+--------------------------------------------------------------------------*/
static const u_char *
VOH_get_clut_pixel_values(
	struct mtvp_voh_data *handler_data
)
{

	/* Color LookUp Table (palette) handling not implemented in this example */

	return NULL;
}

/*--------------------------------------------------------------------------+
| VOH_register
+--------------------------------------------------------------------------*/
int
VOH_register(MTVP_VOH_EXPOSED *exposed, int version)
{
	if (version == MTVP_VOH_VERSION) {

		exposed->VOH_version = MTVP_VOH_VERSION;
		exposed->VOH_init = VOH_init;
		exposed->VOH_free = VOH_free;
		exposed->VOH_window_create = VOH_window_create;
		exposed->VOH_window_destroy = VOH_window_destroy;
		exposed->VOH_resolve_dither = VOH_resolve_dither;
		exposed->VOH_read_and_handle_all_pending_events = VOH_read_and_handle_all_pending_events;
		exposed->VOH_dithered_image_alloc = VOH_dithered_image_alloc;
		exposed->VOH_dithered_image_free = VOH_dithered_image_free;
		exposed->VOH_dithered_image_show = VOH_dithered_image_show;
		exposed->VOH_dithered_image_lock = VOH_dithered_image_lock;
		exposed->VOH_dithered_image_unlock = VOH_dithered_image_unlock;
		exposed->VOH_set_clut = VOH_set_clut;
#ifdef WIN32
		exposed->VOH_realize_clut = NULL;
#endif
		exposed->VOH_free_clut = VOH_free_clut;
		exposed->VOH_get_clut_pixel_values = VOH_get_clut_pixel_values;
		exposed->VOH_get_rgb_masks = VOH_get_rgb_masks;
		exposed->VOH_set_control = VOH_set_control;
		exposed->VOH_cntl = VOH_cntl;
	}
	return MTVP_VOH_VERSION;
}


/* eof */
